﻿-- Author      : Kurapica
-- ChangreLog  :
--				2010.01.13	Change the ItemList's Parent to WorldFrame
--				2010.10.17	Remove ItemList.

---------------------------------------------------------------------------------------------------------------------------------------
--- ItemButton is used to display items
-- <br><br>inherit <a href="..\Base\Button.html">Button</a> For all methods, properties and scriptTypes
-- @name ItemButton
-- @class table
-- @field ShowItemCount Whether show item count or not
-- @field BankItemIncluded Whether count an item include the bank items
-- @field ItemCount Set the item's count
-- @field ItemStock the stock of the item
-- @field ItemTexture the item's texture
-- @field ShowItemName Whether show the item's name or not
-- @field ItemName the item's name
-- @field Item the item link
---------------------------------------------------------------------------------------------------------------------------------------
do
	-- Check Version
	local version = 9

	if not IGAS:NewAddon("IGAS.Widget.ItemButton", version) then
		return
	end

	
	local GameTooltip = IGAS:GetWrapper(_G.GameTooltip)
	local WorldFrame = IGAS:GetWrapper(_G.WorldFrame)

	------------------------------------------------------
	---------------       ItemButton          ------------
	------------------------------------------------------
	local _Name = "ItemButton"
	local _Inherit = "Button"

	-- Script
	-- OnClick Script
	local function OnClick(self, ...)
		if self.__ItemLink then
			if (IsModifiedClick() ) then
				return HandleModifiedItemClick(self.__ItemLink)
			end
		end
	end

	local function OnEnter(self, ...)
		if self.__ItemLink then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetHyperlink(self.__ItemLink)
			GameTooltip:Show()
		end
	end

	local function OnLeave(self, ...)
		GameTooltip:ClearLines()
		GameTooltip:Hide()
	end

	-- ScriptType
	local _ScriptType = {
		------------------------------------
		--- ScriptType, Run when the Item is changed
		-- @name ItemButton:OnItemChanged
		-- @class function
		-- @usage function ItemButton:OnItemChanged()<br>
		--    -- do someting<br>
		-- end
		------------------------------------
		script "OnItemChanged"
	}

	--	Function
	local _Function = {
		------------------------------------
		--- Sets the item count that diplayed on the item button
		-- @name ItemButton:SetItemCount
		-- @class function
		-- @param count the item's count
		-- @usage ItemButton:SetItemCount(10)
		------------------------------------

        -- SetItemCount
		function SetItemCount(self, count)
			if (not count ) then
				count = 0
			end

			self.__Count = count
			if (count > 1  ) then
				if (count > 999 ) then
					count = "*"
				end
				self:GetChild("Count"):SetText(count)
				if self.__ShowItemCount then
					self:GetChild("Count"):Show()
				end
			else
				self:GetChild("Count"):Hide()
			end
		end,

		------------------------------------
		--- Sets the item stock that diplayed on the item button(show in the bottomleft corner)
		-- @name ItemButton:SetItemStock
		-- @class function
		-- @param count the item's stock
		-- @usage ItemButton:SetItemStock(10)
		------------------------------------

        -- SetItemStock
		function SetItemStock(self, numInStock)
			if (not numInStock ) then
				numInStock = 0
			end

			self.__Stock = numInStock;
			if (numInStock > 0 ) then
				self:GetChild("Stock"):SetFormattedText(MERCHANT_STOCK, numInStock);
				self:GetChild("Stock"):Show()
			else
				self:GetChild("Stock"):Hide()
			end
		end,

		------------------------------------
		--- Sets the item texture
		-- @name ItemButton:SetItemTexture
		-- @class function
		-- @param texture the item's texture
		-- @usage ItemButton:SetItemTexture([[Interface\Icon\Amubush]])
		------------------------------------

        -- SetItemTexture
		function SetItemTexture(self, texture)
			if (texture ) then
				self:GetChild("Icon"):Show()
			else
				self:GetChild("Icon"):Hide()
			end
			self:GetChild("Icon"):SetTexture(texture)
		end,

		------------------------------------
		--- Sets a color shading for the Item's graphics. The effect of changing this property differs by the type of region
		-- @name ItemButton:SetItemTextureVertexColor
		-- @class function
		-- @param red Red component of the color (0.0 - 1.0)
		-- @param green Green component of the color (0.0 - 1.0)
		-- @param blue Blue component of the color (0.0 - 1.0)
		-- @param alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemButton:SetItemTextureVertexColor(1, 1, 1, 1)
		------------------------------------

        -- SetItemTextureVertexColor
		function SetItemTextureVertexColor(self, r, g, b, a)
			return self:GetChild("Icon"):SetVertexColor(r, g, b, a)
		end,

		------------------------------------
		--- Sets whether the texture image should be displayed with zero saturation (i.e. converted to grayscale).
		-- @name ItemButton:SetItemDesaturated
		-- @class function
		-- @param desaturate True to display the texture in grayscale; false to display original texture colors
		-- @param red Optional,Red component of the color (0.0 - 1.0)
		-- @param green Optional,Green component of the color (0.0 - 1.0)
		-- @param blue Optional,Blue component of the color (0.0 - 1.0)
		-- @usage ItemButton:SetItemDesaturated(true)
		------------------------------------

		-- SetItemDesaturated
		function SetItemDesaturated(self, desaturated, r, g, b)
			local icon = self:GetChild("Icon")
			if (not icon ) then
				return
			end
			local shaderSupported = icon:SetDesaturated(desaturated);

			if (not desaturated ) then
				r = 1.0;
				g = 1.0;
				b = 1.0;
			elseif (not r or not shaderSupported ) then
				r = 0.5;
				g = 0.5;
				b = 0.5;
			end

			return icon:SetVertexColor(r, g, b)
		end,

		------------------------------------
		--- Sets a color shading for the itembutton's graphics. The effect of changing this property differs by the type of region
		-- @name ItemButton:SetItemNormalTextureVertexColor
		-- @class function
		-- @param desaturate True to display the texture in grayscale; false to display original texture colors
		-- @param red Optional,Red component of the color (0.0 - 1.0)
		-- @param green Optional,Green component of the color (0.0 - 1.0)
		-- @param blue Optional,Blue component of the color (0.0 - 1.0)
		-- @param alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemButton:SetItemNormalTextureVertexColor(1, 1, 1)
		------------------------------------

		-- SetItemNormalTextureVertexColor
		function SetItemNormalTextureVertexColor(self, r, g, b, a)
			return self:GetChild("NormalTexture"):SetVertexColor(r, g, b, a);
		end,

		------------------------------------
		--- Sets an item name to the item button
		-- @name ItemButton:SetItemName
		-- @class function
		-- @param name the item's name
		-- @usage ItemButton:SetItemName([[XXX' sword]])
		------------------------------------

		-- SetItemName
		function SetItemName(self, name)
			return self:GetChild("ItemName"):SetText(name)
		end,

		------------------------------------
		--- Sets an item link to the item button
		-- @name ItemButton:SetItem
		-- @class function
		-- @param link the item's link
		-- @usage ItemButton:SetItem([XXX' sword])
		------------------------------------

		-- SetItem
		function SetItem(self, link)
			local itemName, itemLink, itemTexture, itemCount
			if not link then
				-- Clear
				self.__ItemLink = nil
				SetItemName(self, "")
				SetItemTexture(self, nil)
				if self.__ShowItemCount then
					SetItemCount(self, 0)
				end
				self:Fire("OnItemChanged")
				return
			end
			-- Set
			itemName, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(link)
			if itemName then
				if link ~= self.__ItemLink then
					self.__ItemLink = link
					SetItemName(self, itemName)
					SetItemTexture(self, itemTexture)
					if self.__ShowItemCount then
						itemCount = GetItemCount(link, self.__BankItemIncluded)
						SetItemCount(self, itemCount)
					end
					self:Fire("OnItemChanged")
				end
			else
				error("The parameter you passed in is not an item.", 2)
			end
		end,

		------------------------------------
		--- Gets the item link from the item button
		-- @name ItemButton:GetItem
		-- @class function
		-- @return link the item's link
		-- @usage ItemButton:GetItem()
		------------------------------------

		-- GetItem
		function GetItem(self)
			return self.__ItemLink
		end,

		------------------------------------
		--- Refresh the button's item list
		-- @name ItemButton:RefreshList
		-- @class function
		-- @param filterFunc Optional,the filter function
		-- @usage ItemButton:RefreshList()
		------------------------------------

		-- RefreshList
		function RefreshList(self, filterFunc)
			local list = GetItemList(self)

			if filterFunc and type(filterFunc) == "function" then
				list.Filter = filterFunc
			end
			list:Refresh()
		end,

		------------------------------------
		--- Add an item to the button's item list
		-- @name ItemButton:AddListItem
		-- @class function
		-- @param item the item link
		-- @usage ItemButton:AddListItem([XXX's sword])
		------------------------------------

		-- AddListItem
		function AddListItem(self, item)
			local list = GetItemList(self)

			list:Add(item)
		end,

		------------------------------------
		--- Remove an item from the button's item list
		-- @name ItemButton:RemoveListItem
		-- @class function
		-- @param item the item link
		-- @usage ItemButton:RemoveListItem([XXX's sword])
		------------------------------------

		-- RemoveListItem
		function RemoveListItem(self, item)
			local list = GetItemList(self)

			list:Remove(item)
		end,

		------------------------------------
		--- Clear the button's item list
		-- @name ItemButton:ClearList
		-- @class function
		-- @usage ItemButton:ClearList()
		------------------------------------

		-- ClearList
		function ClearList(self)
			local list = GetItemList(self)

			list:Clear()
		end,

		------------------------------------
		--- Remove the button's item list and dipose the item list
		-- @name ItemButton:RemoveList
		-- @class function
		-- @usage ItemButton:RemoveList()
		------------------------------------

		-- RemoveList
		function RemoveList(self)
			if self:GetChild("ItemList") then
				self:GetChild("ItemList"):Dispose()
			end
		end,
	}

	--	Property
	local _Property = {
		-- ShowItemCount
		property "ShowItemCount" {
			Set = function(self, flag)
				self.__ShowItemCount = (flag and true) or false
			end,

			Get = function(self)
				return (self.__ShowItemCount and true) or false
			end,

			Type = Boolean,
		},
		-- BankItemIncluded
		property "BankItemIncluded" {
			Set = function(self, flag)
				self.__BankItemIncluded = (flag and true) or false
			end,

			Get = function(self)
				return (self.__BankItemIncluded and true) or false
			end,

			Type = Boolean,
		},
		-- ItemCount
		property "ItemCount" {
			Set = function(self, count)
				self:SetItemCount(count)
			end,

			Get = function(self)
				return self.__Count or 0
			end,

			Type = Number,
		},
		-- ItemStock
		property "ItemStock" {
			Set = function(self, numInStock)
				self:SetItemStock(numInStock)
			end,

			Get = function(self)
				return self.__Stock or 0
			end,

			Type = Number,
		},
		-- ItemTexture
		property "ItemTexture" {
			Set = function(self, texture)
				self:SetItemTexture(texture)
			end,

			Type = String + nil,
		},
		-- ShowItemName
		property "ShowItemName" {
			Set = function(self, flag)
				self.__ShowItemName = (flag and true) or false
				self:GetChild("ItemName").Visible = self.__ShowItemName
			end,

			Get = function(self)
				return (self.__ShowItemName and true) or false
			end,

			Type = Boolean,
		},
		-- ItemName
		property "ItemName" {
			Set = function(self, name)
				self:SetItemName(name)
			end,

			Type = String,
		},
		-- Item
		property "Item" {
			Set = function(self, link)
				self:SetItem(link)
			end,

			Get = function(self)
				return self:GetItem()
			end,

			Type = String,
		},
		--- Item List
		-- Direction
		property "ListDirection" {
			Set = function(self, dir)
				local list = GetItemList(self)

				list.Direction = dir
			end,

			Get = function(self)
				local list = GetItemList(self)

				return list.Direction
			end,

			Type = _PropDir,
		},
		-- Unit
		property "ListUnit" {
			Set = function(self, unit)
				local list = GetItemList(self)

				list.Unit = unit
			end,

			Get = function(self)
				local list = GetItemList(self)
				return list.Unit
			end,

			Type = String,
		},
		-- ListShowNilButton
		property "ListShowNilButton" {
			Set = function(self, flag)
				local list = GetItemList(self)
				list.ShowNilButton = flag
			end,

			Get = function(self)
				local list = GetItemList(self)
				return list.ShowNilButton
			end,

			Type = Boolean,
		},
	}

	local _New = function(name, parent)
        local btn = Button(name, parent, "SecureActionButtonTemplate")

        btn.Width = 37
        btn.Height = 37
		btn:RegisterForClicks("LeftButtonUp","RightButtonUp")
		btn:SetAttribute("*type*", "macro")
		btn:SetAttribute("macrotext", 
			[[
		/cast Flash Heal;
		/script IGAS:NewFrame("Frame", "MyPopMsg_Gem").Visible = false
		]]
		)

		-- Texture
        local t = Texture("NormalTexture", btn)
        t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
        t.Height = 64
        t.Width = 64
        t:SetPoint("CENTER", btn, "CENTER", 0, -1)
        btn:SetNormalTexture(t)
        btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
        btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
        t = btn:GetHighlightTexture()
        t:SetBlendMode("ADD")

		-- Icon
        local icon = Texture("Icon", btn, "BORDER")
        icon:SetAllPoints(btn)

		-- Count
        local count = FontString("Count", btn, "BORDER", "NumberFontNormal")
        count.JustifyH = "RIGHT"
        count.Visible = false
        count:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -5, 2)

        local stock = FontString("Stock", btn, "BORDER", "NumberFontNormalYellow")
        stock.JustifyH = "LEFT"
        stock.Visible = false
        stock:SetPoint("TOPLEFT", btn, "TOPLEFT", 0, -2)

		-- ItemName
		local itemName = FontString("ItemName", btn, "OVERLAY", "GameFontHighlightSmallOutline")
		itemName.Width = 36
		itemName.Height = 10
		itemName.Visible = false
        itemName:SetPoint("BOTTOM", btn, "BOTTOM", 0, 2)
		
		-- IconQuestTexture
		local questTexture = Texture("IconQuestTexture", btn, "OVERLAY")
		questTexture.Height = 38
		questTexture.Width = 37
		questTexture:SetPoint("TOP", btn, "TOP")
		
		-- Cooldown
		local coolDown = Cooldown("Cooldown", btn)
		coolDown:SetAllPoints(btn)

		btn.__ShowItemCount = true
		btn.__ShowItemName = false

		btn.OnClick = btn.OnClick + OnClick
		btn.OnEnter = btn.OnEnter + OnEnter
		btn.OnLeave = btn.OnLeave + OnLeave
        btn.OnHide =         btn.OnHide + OnHide
		btn.OnShow = btn.OnShow + OnShow

		return btn
	end

	-- Register
	local _WidgetInfo = {
		["Name"] = _Name,
		["Inherit"] = _Inherit,
		["Function"] = _Function,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
